@@ -1,7 +1,7 @@
 package File::Write::Rotate;
 
-our $DATE = '2015-01-23'; # DATE
-our $VERSION = '0.27'; # VERSION
+our $DATE = '2015-01-26'; # DATE
+our $VERSION = '0.28'; # VERSION
 
 use 5.010001;
 use strict;
@@ -21,36 +21,58 @@ our $Debug;
 
 sub new {
     my $class = shift;
-    my %args  = @_;
+    my %args0 = @_;
 
-    defined( $args{dir} )    or croak "Please specify dir";
-    defined( $args{prefix} ) or croak "Please specify prefix";
-    $args{suffix} //= "";
+    my %args;
 
-    $args{size} //= 0;
+    defined($args{dir} = delete $args0{dir})
+        or croak "Please specify dir";
+    defined($args{prefix} = delete $args0{prefix})
+        or croak "Please specify prefix";
+    $args{suffix} = delete($args0{suffix}) // "";
 
-    if ( $args{period} ) {
-        $args{period} =~ /daily|day|month|year/i
-          or croak "Invalid period, please use daily/monthly/yearly";
+    $args{size} = delete($args0{size}) // 0;
+
+    $args{period} = delete($args0{period});
+    if ($args{period}) {
+        $args{period} =~ /\A(daily|day|month|monthly|year|yearly)\z/
+            or croak "Invalid period, please use daily/monthly/yearly";
     }
 
-    for (grep {/\Ahook_/} keys %args) {
+    for (map {"hook_$_"} qw(before_rotate after_rotate after_create
+                            before_write a)) {
+        next unless $args0{$_};
+        $args{$_} = delete($args0{$_});
         croak "Invalid $_, please supply a coderef"
             unless ref($args{$_}) eq 'CODE';
     }
 
-    if ( !$args{period} && !$args{size} ) {
+    if (!$args{period} && !$args{size}) {
         $args{size} = 10 * 1024 * 1024;
     }
 
-    $args{histories} //= 10;
+    $args{histories} = delete($args0{histories}) // 10;
+
+    $args{binmode} = delete($args0{binmode});
+
+    $args{buffer_size} = delete($args0{buffer_size});
+
+    $args{lock_mode} = delete($args0{lock_mode}) // 'write';
+    $args{lock_mode} =~ /\A(none|write|exclusive)\z/
+        or croak "Invalid lock_mode, please use none/write/exclusive";
+
+    $args{rotate_probability} = delete($args0{rotate_probability});
+    if (defined $args{rotate_probability}) {
+        $args{rotate_probability} > 0 && $args{rotate_probability} < 1.0
+            or croak "Invalid rotate_probability, must be 0 < x < 1";
+    }
+
+    if (keys %args0) {
+        croak "Unknown arguments to new(): ".join(", ", sort keys %args0);
+    }
 
     $args{_buffer} = [];
 
-    $args{lock_mode} //= 'write';
-    $args{lock_mode} =~ /\A(none|write|exclusive)\z/
-      or croak "Invalid lock_mode, please use none/write/exclusive";
-
     my $self = bless \%args, $class;
 
     $self->{_exclusive_lock} = $self->_get_lock
@@ -65,8 +87,7 @@ sub buffer_size {
         my $old = $self->{buffer_size};
         $self->{buffer_size} = $_[0];
         return $old;
-    }
-    else {
+    } else {
         return $self->{buffer_size};
     }
 }
@@ -88,23 +109,20 @@ sub _file_path {
     # _now is calculated every time we access this method
     $self->{_now} = time();
 
-    my @lt = localtime( $self->{_now} );
+    my @lt = localtime($self->{_now});
     $lt[5] += 1900;
     $lt[4]++;
 
     my $period;
-    if ( $self->{period} ) {
-        if ( $self->{period} =~ /year/i ) {
-            $period = sprintf( "%04d", $lt[5] );
+    if ($self->{period}) {
+        if ($self->{period} =~ /year/i) {
+            $period = sprintf("%04d", $lt[5]);
+        } elsif ($self->{period} =~ /month/) {
+            $period = sprintf("%04d-%02d", $lt[5], $lt[4]);
+        } elsif ($self->{period} =~ /day|daily/) {
+            $period = sprintf("%04d-%02d-%02d", $lt[5], $lt[4], $lt[3]);
         }
-        elsif ( $self->{period} =~ /month/i ) {
-            $period = sprintf( "%04d-%02d", $lt[5], $lt[4] );
-        }
-        elsif ( $self->{period} =~ /day|daily/i ) {
-            $period = sprintf( "%04d-%02d-%02d", $lt[5], $lt[4], $lt[3] );
-        }
-    }
-    else {
+    } else {
         $period = "";
     }
 
@@ -133,7 +151,7 @@ sub _get_lock {
     return $self->{_weak_lock} if defined($self->{_weak_lock});
 
     require File::Flock::Retry;
-    my $lock = File::Flock::Retry->lock( $self->lock_file_path );
+    my $lock = File::Flock::Retry->lock($self->lock_file_path);
     $self->{_weak_lock} = $lock;
     weaken $self->{_weak_lock};
     return $lock;
@@ -151,7 +169,7 @@ sub _get_files {
     };
 
     my @files;
-    while ( my $e = readdir($dh) ) {
+    while (my $e = readdir($dh)) {
         my $cs = $1 if $e =~ s/(\.gz)\z//; # compress suffix
         next unless $e =~ /\A\Q$self->{prefix}\E
                            (?:\. (?<period>\d{4}(?:-\d\d(?:-\d\d)?)?) )?
@@ -183,7 +201,7 @@ sub _rotate_and_delete {
         #
         # XXX check validity of PID file, otherwise a stale PID file will always
         # prevent rotation to be done
-        if ( -f "$self->{dir}/$self->{prefix}-compress.pid" ) {
+        if (-f "$self->{dir}/$self->{prefix}-compress.pid") {
             warn "Compression is in progress, rotation is postponed";
             last CASE;
         }
@@ -198,7 +216,7 @@ sub _rotate_and_delete {
         my $dir = $self->{dir};
         my $rotating_period = @$files ? $files->[-1][2] : undef;
         for my $f (@$files) {
-            my ( $orig, $rs, $period, $cs ) = @$f;
+            my ($orig, $rs, $period, $cs) = @$f;
             $i++;
 
             #say "DEBUG: is_tainted \$dir? ".is_tainted($dir);
@@ -209,7 +227,7 @@ sub _rotate_and_delete {
             # case, _get_files)
             untaint \$orig;
 
-            if ( $i <= @$files - $self->{histories} ) {
+            if ($i <= @$files - $self->{histories}) {
                 say "DEBUG: Deleting old rotated file $dir/$orig$cs ..."
                     if $Debug;
                 if (unlink "$dir/$orig$cs") {
@@ -219,15 +237,14 @@ sub _rotate_and_delete {
                 }
                 next;
             }
-            if ( !$delete_only && defined($rotating_period) && $period eq $rotating_period ) {
+            if (!$delete_only && defined($rotating_period) && $period eq $rotating_period) {
                 my $new = $orig;
                 if ($rs) {
                     $new =~ s/\.(\d+)\z/"." . ($1+1)/e;
-                }
-                else {
+                } else {
                     $new .= ".1";
                 }
-                if ( $new ne $orig ) {
+                if ($new ne $orig) {
                     say "DEBUG: Renaming rotated file $dir/$orig$cs -> ".
                         "$dir/$new$cs ..." if $Debug;
                     if (rename "$dir/$orig$cs", "$dir/$new$cs") {
@@ -249,7 +266,7 @@ sub _open {
 
     my ($fp, $period) = $self->_file_path;
     open $self->{_fh}, ">>", $fp or die "Can't open '$fp': $!";
-    if ( defined $self->{binmode} ) {
+    if (defined $self->{binmode}) {
         binmode $self->{_fh}, $self->{binmode}
           or die "Can't set PerlIO layer on '$fp': $!";
     }
@@ -264,13 +281,19 @@ sub _open {
 sub _rotate_and_open {
 
     my $self = shift;
-    my ( $do_open, $do_rotate ) = @_;
-    my $fp = $self->_file_path;
-    my %rotate_params = ();
+    my ($do_open, $do_rotate) = @_;
+    my $fp;
+    my %rotate_params;
 
   CASE:
     {
-        unless ( -e $fp ) {
+        # if instructed, only do rotate some of the time to shave overhead
+        if ($self->{rotate_probability} && $self->{_fh}) {
+            last CASE if rand() > $self->{rotate_probability};
+        }
+
+        $fp = $self->_file_path;
+        unless (-e $fp) {
             $do_open++;
             $do_rotate++;
             $rotate_params{delete_only} = 1;
@@ -278,12 +301,12 @@ sub _rotate_and_open {
         }
 
         # file is not opened yet, open
-        unless ( $self->{_fh} ) {
+        unless ($self->{_fh}) {
             $self->_open;
         }
 
         # period has changed, rotate
-        if ( $self->{_fp} ne $fp ) {
+        if ($self->{_fp} ne $fp) {
             $do_rotate++;
             $rotate_params{delete_only} = 1;
             last CASE;
@@ -292,20 +315,18 @@ sub _rotate_and_open {
         # check whether size has been exceeded
         my $inode;
 
-        if ( $self->{size} > 0 ) {
+        if ($self->{size} > 0) {
 
-            my @st   = stat( $self->{_fh} );
+            my @st   = stat($self->{_fh});
             my $size = $st[7];
             $inode = $st[1];
 
-            if ( $size >= $self->{size} ) {
+            if ($size >= $self->{size}) {
                 say "DEBUG: Size of $self->{_fp} is $size, exceeds $self->{size}, rotating ..."
                     if $Debug;
                 $do_rotate++;
                 last CASE;
-            }
-            else {
-
+            } else {
                 # stat the current file (not our handle _fp)
                 my @st = stat($fp);
                 die "Can't stat '$fp': $!" unless @st;
@@ -314,10 +335,9 @@ sub _rotate_and_open {
                 # check whether other process has rename/rotate under us (for
                 # example, 'prefix' has been moved to 'prefix.1'), in which case
                 # we need to reopen
-                if ( ( defined($inode) ) and ( $finode != $inode ) ) {
+                if (defined($inode) && $finode != $inode) {
                     $do_open++;
                 }
-
             }
 
         }
@@ -336,7 +356,7 @@ sub write {
     # FYI: if privilege is dropped from superuser, the failure is usually at
     # locking the lock file (permission denied).
 
-    my @msg = ( map( {@$_} @{ $self->{_buffer} } ), @_ );
+    my @msg = (map( {@$_} @{ $self->{_buffer} } ), @_);
 
     eval {
         my $lock = $self->_get_lock;
@@ -355,12 +375,10 @@ sub write {
     my $err = $@;
 
     if ($err) {
-        if ( ( $self->{buffer_size} // 0 ) > @{ $self->{_buffer} } ) {
-
+        if (($self->{buffer_size} // 0) > @{ $self->{_buffer} }) {
             # put message to buffer temporarily
             push @{ $self->{_buffer} }, [@_];
-        }
-        else {
+        } else {
             # buffer is already full, let's dump the buffered + current message
             # to the die message anyway.
             die join(
@@ -369,7 +387,7 @@ sub write {
                 (
                     @{ $self->{_buffer} }
                     ? " (buffer is full, "
-                      . scalar( @{ $self->{_buffer} } )
+                      . ~~@{ $self->{_buffer} }
                       . " message(s))"
                     : ""
                 ),
@@ -389,7 +407,7 @@ sub compress {
     my $files_ref        = $self->_get_files;
     my $done_compression = 0;
 
-    if ( @{$files_ref} ) {
+    if (@{$files_ref}) {
         my $pid = Proc::PID::File->new(
             dir    => $self->{dir},
             name   => "$self->{prefix}-compress",
@@ -397,23 +415,23 @@ sub compress {
         );
         my $latest_period = $files_ref->[-1][2];
 
-        if ( $pid->alive ) {
+        if ($pid->alive) {
             warn "Another compression is in progress";
         } else {
             my @tocompress;
             #use DD; dd $self;
-            for my $file_ref ( @{$files_ref} ) {
+            for my $file_ref (@{$files_ref}) {
                 my ($orig, $rs, $period, $cs) = @{ $file_ref };
                 #say "D:compress: orig=<$orig> rs=<$rs> period=<$period> cs=<$cs>";
                 next if $cs; # already compressed
                 next if !$self->{period} && !$rs; # not old file
                 next if  $self->{period} && $period eq $latest_period; # not old file
-                push @tocompress, File::Spec->catfile( $self->{dir}, $orig );
+                push @tocompress, File::Spec->catfile($self->{dir}, $orig);
             }
 
             if (@tocompress) {
-                foreach my $file (@tocompress) {
-                    gzip( $file => "$file.gz" )
+                for my $file (@tocompress) {
+                    gzip($file => "$file.gz")
                         or do { warn "gzip failed: $GzipError\n"; next };
                     unlink $file;
                 }
@@ -449,7 +467,7 @@ File::Write::Rotate - Write to files that archive/rotate themselves
 
 =head1 VERSION
 
-This document describes version 0.27 of File::Write::Rotate (from Perl distribution File-Write-Rotate), released on 2015-01-23.
+This document describes version 0.28 of File::Write::Rotate (from Perl distribution File-Write-Rotate), released on 2015-01-26.
 
 =head1 SYNOPSIS
 
@@ -560,6 +578,8 @@ This hook can be used to write a header to each file, e.g.:
 
 Since this is called indirectly by write(), locking is also already done.
 
+=head2 binmode => str
+
 =head1 METHODS
 
 =head2 $obj = File::Write::Rotate->new(%args)
@@ -664,6 +684,14 @@ acquire lock, will die if failed to acquire lock.
 
 See L</ATTRIBUTES>.
 
+=item * buffer_size => int
+
+=item * rotate_probability => float (between 0 < x < 1)
+
+If set, instruct to only check for rotation under a certain probability, for
+example if value is set to 0.1 then will only check for rotation 10% of the
+time.
+
 =back
 
 =head2 lock_file_path => STR
@@ -703,7 +731,7 @@ misconfigured will cause your logs to stop being rotated and grow indefinitely).
 =head2 What is the downside of using FWR (and LDFR)?
 
 Mainly (significant) performance overhead. At (almost) every C<write()>, FWR
-needs to check file sizes and dates for rotation. Under default configuration
+needs to check file sizes and/or dates for rotation. Under default configuration
 (where C<lock_mode> is C<write>), it also performs locking on each C<write()> to
 make it safe to use with multiple processes. Below is a casual benchmark to give
 a sense of the overhead, tested on my Core i5-2400 3.1GHz desktop:
@@ -715,6 +743,15 @@ C<lock_mode> C<none> or C<exclusive>, the speed is ~52k/s.
 However, this is not something you'll notice or need to worry about unless
 you're writing near that speed.
 
+If you need more speed, you can try setting C<rotate_probability> which will
+cause FWR to only check for rotation probabilistically, e.g. if you set this to
+0.1 then checks will only be done in about 1 of 10 writes. This can
+significantly reduce the overhead and increase write speed several times (e.g.
+5-8 times), but understand that this will make the writes "overflow" a bit, e.g.
+file sizes will exceed for a bit if you do size-based rotation. More suitable if
+you only do size-based rotation since it is usually okay to exceed sizes for a
+bit.
+
 =head1 SEE ALSO
 
 L<Log::Dispatch::FileRotate>, which inspires this module. Differences between
